#ifndef ROOM_H
#define ROOM_H 1

  /*\
   *  $Header: room.h,v 5.1 90/01/11 13:39:18 erik Exp $
   *
   *		              COPYRIGHT 1990
   *		        DIGITAL EQUIPMENT CORPORATION
   *		           MAYNARD, MASSACHUSETTS
   *			    ALL RIGHTS RESERVED.
   *
   * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
   * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
   * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE 
   * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED 
   * WARRANTY.
   *
   * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
   * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
   * ADDITION TO THAT SET FORTH ABOVE.
   *
   * Permission to use, copy, modify, and distribute this software and its
   * documentation for any purpose and without fee is hereby granted, provided
   * that the above copyright notice appear in all copies and that both that
   * copyright notice and this permission notice appear in supporting
   * documentation, and that the name of Digital Equipment Corporation not be
   * used in advertising or publicity pertaining to distribution of the 
   * software without specific, written prior permission.
  \*/

  /*\

  \*/

	/*\
	 *  Methods for manipulating a Room.    A Room represents a single
	 *  room, presumably in a rooms application.   This package deals
	 *  with rooms in isolation;  it does not deal with interactions
	 *  or between multiple rooms.
	 *
	 *  A room has a set of Apps whose geometry it manages.   An App is
	 *  local to a Room if the App has a room state specific to that 
	 *  Room.   A Room may manage Apps that are not local to the Room,
	 *  in which case the Room should use the App default state.  The
	 *  "visible" state of an App in a Room is the state that the Room 
	 *  tries to maintain for that App (i.e.  the local state of the App
	 *  if the App is local to the Room, the default state of the App if
	 *  the App is managed but not local, and NullAppState if the App is
	 *  not managed by the room).
	 *
	 *  An active room tries to change the current state of the Apps it
	 *  manages whenever that current state is out of sync with the
	 *  visible state of the App in the Room.  An inactive Room allow
	 *  the App's current state to diverge from the Room's visible
	 *  state for that App.
	 *
	 *  App's may be managed by multiple rooms, but no more than one
	 *  of the managing rooms should be active at any time or chaos
	 *  may result.
	 *
	 *  A Room may exist without being visible to the user (of xrooms); 
	 *  such a Room is "hidden."   Any Room that is visible to the user
	 *  is said to be "visible."   Note that it is quite confusing to the 
	 *  user for an active Room to be hidden.    Room visibility may be 
	 *  controlled explicitly or automatically.   
	 *  The explicit visibility controls are:
	 *	roomVisible	 --  Room is visible.
	 *      roomHidden       --  Room is hidden.
	 *  The automatic visibility controls that this package provides are:  
	 *	roomAlways	 --  Room is always visible.
	 *	roomWhenActive   --  Room is visible whenever it is active.
	 *	roomWhenNonEmpty --  Room is visible whenever it has at least
	 *			     one *local* App.
	 *  Any combination of automatic controls can be specified.
	 *
	 *  This package was written to support an implementation of rooms
	 *  under X, but it doesn't know anything about X.   I relies on
	 *  the App package to shove applications around and calls a
	 *  Feedback function supplied by xrooms whenever the state of the
	 *  room changes.   This function should provide any feedback xrooms 
	 *  wants to give the user (highlighting the button of the active room 
	 *  or adding the button of a newly visible room, for example).
	 *  The types of feedback reported to xrooms are:
	 *	roomCreated, roomDestroyed, roomActivated, roomDeactivated,
	 *	roomHidden, roomVisible, roomAppAdded, roomAppRemoved
	 *
	 *  PUBLIC PACKAGE GLOBAL VARIABLE --
	 *  unsigned	roomDebug;
	 *	Set of debugging flags.  The only flags currently defined
	 *	is ENTER (0x10) defined in ../utils/utils.h.  Intial value
	 *	is 0.
	 *  unsigned	roomDefaultAppSlots;
	 *	The number of slots allocated for Apps in a Room by default.
	 *	App slots are increased dynamically, so this value isn't
	 *	too critical.   It should be a reasonable guesstimate of the
	 *	maximum number of Apps *active* at any one time to help
	 *	minimize allocator overhead and fragmentation.  Initial
	 *	value is 17.
	 *  unsigned	roomDefaultVisibility;
	 *	Default visibility of newly created Rooms.   Initial value
	 *	is (roomWhenActive|roomWhenNonEmpty).
	 *  Boolean	roomIconifyFirst;
	 *	If True, iconify all applications that need to be iconified
	 *	before moving or opening other applications.   If False,
	 *	order is undefined.   Default is True.
	 *
	 *  DEBUGGGING FUNCTIONS --
	 *  char *
	 *  roomText(pRoom)
	 *  RoomPtr	pRoom;
	 *	Returns the name of "pRoom" in a static buffer which may be
	 *	clobbered by subsequent calls to roomText.   Guaranteed not
	 *	to return NullString.
	 *
	 *  void
	 *  roomDebugPrint(pRoom)
	 *	Prints scads of useful debugging information about "pRoom"
	 *	using uDebug (which usually writes to stderr).
	 *
	 *  FUNCTIONS TO CREATE AND DESTROY ROOMS --
	 *  RoomPtr
	 *  roomCreate(name,private,feedback)
	 *  StringToken		name;
	 *  Opaque		private;
	 *  RoomFeedbackFunc	feedback.
	 *	Creates a room with the specified name, private field and
	 *	feedback function.   The new Room manages no applications
	 *	and has default visibility.   
	 *
	 *	Calls "feedback" (if non-NULL) to note "roomCreated."
	 *
	 *	Returns a pointer to the newly created Room or NullRoom 
	 *	if name is NullStringToken or there is an allocation failure.
	 *
	 *  void
	 *  roomDestroy(pRoom)
	 *  RoomPtr	pRoom;
	 *	Destroys "pRoom" and frees any memory it uses.  roomDestroy
	 *	first deactivates the the Room if active, then hides the
	 *	room if visible, and then finally destroys the room and 
	 *	frees any memory it uses.   The feedback function is called
	 *	separately for the deactivation, the hide and the destroy
	 *	as appropriate.   No return value.
	 *
	 *  FUNCTIONS TO CHANGE OR QUERY THE SET OF MANAGED APPLICATIONS --
	 *  Boolean
	 *  roomManageApp(pRoom,pApp)
	 *  RoomPtr	pRoom;
	 *  AppPtr	pApp;
	 *	Start managing "pApp" in "pRoom."    Queries "pApp" for  
	 *	local, default, and current state.  
	 *	
	 *	May call the feedback function if "pRoom" becomes visible
	 *	because it is no longer empty.
	 *
	 *	Calls the feedback function to note "roomAppAdded"
	 *
	 *	Returns True for success, False if "pRoom" is NullRoom, 
	 *	"pApp" is NullApp, or if "pApp"  (or another App with the 
	 *	same name) is already managed in "pRoom."  A Room may *not* 
	 *	manage multiple Apps with the same name.
	 *
	 *  Boolean
	 *  roomUnmanageApp(pRoom,pApp)
	 *  RoomPtr	pRoom;
	 *  AppPtr	pApp;
	 *	Stop managing "pApp" in "pRoom."  
	 *
	 *	Calls the feedback function to note "roomAppRemoved"
	 *
	 *	May call the feedback function if "pRoom" is hidden because
	 *	the last local App is removed.
	 *
	 *	Returns True for success, False if "pRoom" is NullRoom, 
	 *	"pApp" is NullApp, or "pApp" is not being managed by "pRoom."
	 *
	 *  AppPtr
	 *  roomLookupManagedApp(pRoom,appName)
	 *  RoomPtr	pRoom;
	 *  StringToken	appName;
	 *	Returns a pointer to the App named "appName" being managed
	 *	by "pRoom," or NullApp if "pRoom" is NullRoom, "appName"
	 *	is NullStringToken, or "pRoom" does not manage an App named 
	 *	"appName."   A Room may *not* manage more than one App with 
	 *	the same name.
	 *
	 *  FUNCTIONS TO CHANGE, CHECK, OR QUERY THE STATE OF AN APP IN A ROOM 
	 *  Boolean
	 *  roomAppRealStateChanged(pRoom,pApp,pReal)
	 *  RoomPtr	pRoom;
	 *  AppPtr	pApp;
	 *  AppStatePtr	pReal;
	 *	Note that the real state of "pApp" has changed, and that the
	 *	change should be reflected in "pRoom."   If "pApp" has a state
	 *	local to "pRoom", the local state is updated to reflect the
	 *	changes in pReal.   If "pApp" does not have a state local to
	 *	"pRoom,"  roomAppRealStateChanged creates a new local state
	 *	and modifies to reflect the values in "pReal."   
	 *
	 *	Note that any bolted values in the local or default state will 
	 *	*not* be changed to reflect "pReal."  See appstate.h and app.h
	 *	for a more complete discussion of copying or matching <real> 
	 *	and <profile> states and of bolts.
	 *
	 *	If "pRoom's" visibility changes due to a change in the number
	 *	of local apps, roomAppRealStateChanged calls the feedback
	 *	function to note the change.
	 *
	 *	If "pRoom" is active, and "pReal" does not match the new 
	 *	local state (i.e. if any fields in the local state are
	 *	bolted and different than "pReal"), roomAppRealStateChanged 
	 *	(re)sets the current state of "pApp" to reflect the  new
	 *	local state.
	 *
	 *	Returns True for success, False if "pRoom" is NullRoom, 
	 *	"pApp" is NullApp, "pReal" is not a legal <real> state,
	 *	"pApp" is not managed in "pRoom,"  or if an allocation 
	 *	failure occurs.
	 *
	 *  Boolean
	 *  roomChangeAppLocalState(pRoom,pApp,pProf)
	 *  RoomPtr	pRoom;
	 *  AppPtr	pApp;
	 *  AppStatePtr	pProf;
	 *	Change the local state of "pApp" to "pProf."   If "pApp"
	 *	does not have a state local to "pRoom," a new local state is
	 *	created and "pProf" is copied into it.   Note that 
	 *	roomChangeAppLocalState does *not* respect bolts.
	 *
	 *	If "pRoom" is active, and the current real state of "pApp"
	 *	does not match "pProf," roomChangeAppLocalState tries
	 *	to reset the current state to match the new local state.
	 *
	 *	If "pRoom's" visibility changes due to a change in the number
	 *	of local apps, roomChangeAppLocalState calls the feedback
	 *	function to note the change.
	 *	
	 *	Returns True for success, False if "pRoom" is NullRoom,
	 *	"pApp" is NullApp, "pProf" is not a legal <profile>
	 *	state, "pApp" is not managed in "pRoom," or an allocation
	 *	fails.
	 *
	 *  Boolean
	 *  roomCheckAppStates(pRoom,pApp)
	 *  RoomPtr	pRoom;
	 *  AppPtr	pApp;
	 *	Check the current, local, and default states of "pApp" in
	 *	"pRoom" to make sure they are up to date.
	 *
	 *	If "pRoom" is active and the re-synchronized current state
	 *	does not match the visible state, roomCheckAppStates
	 *	updates the current state.
	 *
	 *	Use this function to update a Room whenever some other package 
	 *	has changed "pApp." 
	 *	
	 *	If room visibility changes (due to a change in the number of
	 *	local Apps), roomCheckAppState calls the feedback func.
	 *
	 *	Returns True for success, False if "pRoom" is NullRoom, 
	 *	"pApp" is NullApp, or "pRoom" does not manage "pApp."
	 *
	 *  AppStatePtr
	 *  roomGetVisibleState(pRoom,pApp)
	 *  RoomPtr	pRoom;
	 *  AppPtr	pApp;
	 *	Returns the visible state of "pApp" in "pRoom," or False
	 *	if "pRoom" is NullRoom, "pApp" is NullApp, or "pApp" is
	 *	not managed in "pRoom."
	 *
	 *  FUNCTIONS TO CHANGE/QUERY ROOM VISIBILITY --
	 *  Boolean
	 *  roomIsVisible(pRoom)
	 *  RoomPtr	pRoom;
	 *	Returns True if "pRoom" is visible, False if "pRoom" is
	 *	NullRoom or hidden.
	 *
	 *  Boolean
	 *  roomSetVisibility(pRoom,visibility,check)
	 *  RoomPtr	pRoom;
	 *  Boolean	visibility;
	 *  Boolean	check;
	 *	Change the visibility of "pRoom" to "visibility."  If
	 *	"check" is True, roomSetVisibility checks for the
	 *	automatic visibility controls roomAlways, roomWhenActive,
	 *	and roomWhenNonEmpty and sets Visible/Hidden appropriately. 
	 *	Calls feedback if visibility changes.
	 *	Returns True for success, False if "pRoom"  if NullRoom.
	 *  
	 *  Boolean
	 *  roomCheckVisibility(pRoom)
	 *  RoomPtr	pRoom;
	 *	Check to make sure that the visibility of "pRoom" is
	 *	correct with respect to any automatic visibility controls 
	 *	defined for "pRoom."  If visibility is incorrect, 
	 *	roomCheckVisibility corrects visibility and calls the
	 *	feedback function.
	 *
	 *	Returns True for success, False if "pRoom" is NullRoom.
	 *
	 *  unsigned
	 *  roomGetVisibility(pRoom)
	 *  RoomPtr	pRoom;
	 *	Returns the visibility of "pRoom," or roomHidden if "pRoom"
	 *	is NullRoom.
	 *
	 *  FUNCTIONS TO ACTIVATE/DEACTIVATE A ROOM --
	 *  Boolean
	 *  roomIsActive(pRoom)
	 *  RoomPtr	pRoom;
	 *	Returns True if "pRoom" is not NullRoom and is active, False 
	 *	otherwise.
	 *
	 *  Boolean
	 *  roomActivate(pRoom)
	 *  RoomPtr	pRoom;
	 *	Activates "pRoom."   If "pRoom" is inactive, roomActivate 
	 *	checks the current state of all Apps managed by "pRoom" and 
	 *	changes any that don't match their visible state in "pRoom."
	 *	Calls the feedback function if visibility changes, and again 
	 *	if "pRoom" was not active.
	 *
	 *	Returns True for success, False if "pRoom" is NullRoom.
	 *
	 *  Boolean
	 *  roomDeactivate(pRoom)
	 *  RoomPtr	pRoom;
	 *	Deactivates "pRoom."  Does not change the state of any 
	 *	Apps.   Calls the feedback function if visibility changes and
	 *	if "pRoom" was active.
	 *	Returns True for success, False if "pRoom" is NullRoom.
	 *
	 *
	 *  MISCELLANEOUS ACCESS FUNCTIONS --
	 *  StringToken
	 *  roomName(pRoom)
	 *  RoomPtr	pRoom;
	 *	Returns the name of "pRoom," or NullStringToken if "pRoom"
	 *	is NullRoom.
	 *
	 *  unsigned
	 *  roomNumApps(pRoom)
	 *  RoomPtr	pRoom;
	 *	Returns the number of Apps managed by "pRoom," or 0 if 
	 *	pRoom is NullRoom.
	 *
	 *  unsigned
	 *  roomNumLocalApps(pRoom);
	 *  RoomPtr	pRoom;
	 *	Returns the number of managed Apps that have a state local to
	 *	"pRoom," or 0 if pRoom is NullRoom.
	 *
	 *  void
	 *  roomSetFeedback(pRoom,pFunc)
	 *  RoomPtr		pRoom;
	 *  RoomFeedbackFunc	pFunc;
	 *	Set the feedback function of "pRoom" to "pFunc."   No
	 *	return value.
	 *  
	 *  RoomFeedbackFunc
	 *  roomGetFeedback(pRoom)
	 *  RoomPtr		pRoom;
	 *	Returns the feedback function of "pRoom," or NULL if
	 *	"pRoom" is NullRoom.
	 *
	 *  void
	 *  roomSetPrivate(pRoom,priv)
	 *  RoomPtr	pRoom;
	 *  Opaque	priv;
	 *	Sets the private value associated with "pRoom" to
	 *	"priv."  No return value.
	 *
	 *  Opaque
	 *  roomGetPrivate(pRoom)
	 *  RoomPtr	pRoom;
	 *	Returns the private value associated with "pRoom," or
	 *	NULL if pRoom is NullRoom.
	 *
	 *  ITERATORS 
	 *  void
	 *  roomIterateApps(pRoom,pFunc,arg)
	 *  RoomPtr		pRoom;
	 *  RoomIterFunc	pFunc;
	 *  Opaque		arg;
	 *	Calls "pFunc" with (pApp,pLocal,pDflt,pReal) for each of the 
	 *	Apps managed by "pRoom" in turn.   Stops iterating if
	 *	"pFunc" returns False.
	 *
	 *  void
	 *  roomIterateLocalApps(pRoom,pFunc,arg)
	 *  RoomPtr		pRoom;
	 *  RoomIterFunc	pFunc;
	 *  Opaque		arg;
	 *	Calls "pFunc" with (pApp,pLocal,pDflt,pReal) for each of the 
	 *	Apps with a local state in "pRoom" in turn.   Stops iterating 
	 *	if "pFunc" returns False.
	 *
	\*/

#include "strtbl.h"
#include "app.h"

typedef struct _RoomRec	*RoomPtr;
#define	NullRoom	((RoomPtr)NULL)

typedef	void	(*RoomFeedbackFunc)(/* pRoom, newState, pApp */);
typedef	Boolean	(*RoomIterFunc)(/* pApp, pLocal, pDflt, pReal, arg */);

	/*
	 * Types of feedback
	 */
#define	roomFBCreated		((unsigned)1<<0)
#define	roomFBDestroyed		((unsigned)1<<1)
#define	roomFBActivated		((unsigned)1<<2)
#define	roomFBDeactivated	((unsigned)1<<3)
#define	roomFBHidden		((unsigned)1<<4)
#define	roomFBVisible		((unsigned)1<<5)
#define roomFBAppAdded		((unsigned)1<<6)
#define	roomFBAppRemoved	((unsigned)1<<7)

	/*
	 * Settings for Room visibility
	 */
#define	roomHidden		((unsigned)0)
#define	roomVisible		((unsigned)1<<0)
#define	roomWhenNonEmpty	((unsigned)1<<1)
#define	roomWhenActive		((unsigned)1<<2)
#define	roomAlways		((unsigned)1<<3)

	/*
	 * public package globals
	 */
#define	ROOM_DEFAULT_APP_SLOTS	17
#define	ROOM_DEFAULT_VISIBILITY	(roomWhenActive|roomWhenNonEmpty)
extern	unsigned	roomDebug;
extern	unsigned	roomDefaultAppSlots;
extern	unsigned	roomDefaultVisibility;
extern	Boolean		roomIconifyFirst;

extern	char		*roomText(/* pRoom */);
extern	void		 roomDebugPrint(/* pRoom */);

extern	RoomPtr		roomCreate(/* name, priv, feedback */);
extern	void		roomDestroy(/* pRoom */);

extern	Boolean		roomManageApp(/* pRoom, pApp */);
extern	Boolean		roomUnmanageApp(/* pRoom,pApp */);
extern	AppPtr		roomLookupManagedApp(/*pRoom,appName*/);

extern	Boolean		roomAppRealStateChanged(/* pRoom, pApp, pReal */);
extern	Boolean		roomChangeAppLocalState(/* pRoom, pApp, pProf */);
extern	Boolean		roomCheckAppStates(/* pRoom, pApp */);
extern	AppStatePtr	roomGetVisibleState(/* pRoom, pApp */);

extern	Boolean		roomIsVisible(/* pRoom */);
extern	Boolean		roomSetVisibility(/* pRoom, visibility, check */);
extern	unsigned	roomGetVisibility(/* pRoom */);

extern	Boolean		roomIsActive(/* pRoom */);
extern	Boolean		roomActivate(/* pRoom */);
extern	Boolean		roomDeactivate(/* pRoom */);

extern	StringToken	roomName(/* pRoom */);
extern	unsigned	roomNumApps(/* pRoom */);
extern	unsigned	roomNumLocalApps(/* pRoom */);
extern	void		roomSetFeedback(/* pRoom, pFunc */);
extern	RoomFeedbackFunc roomGetFeedback(/* pRoom */);
extern	void		roomSetPrivate(/* pRoom, priv */);
extern	Opaque		roomGetPrivate(/* pRoom */);

extern	void		roomIterateApps(/* pRoom, pFunc, arg */);
extern	void		roomIterateLocalApps(/* pRoom, pFunc, arg */);
#endif /* ROOM_H */
